
// External Eprom Access. Jamie Balfour. Dec 2002.
// No Liability Accepted.
#include "Eprom_Functions.h"
#include <system.h>
#include <i2c_driver.h>
 
//NOTE for 16F88 target, you may need to change the delay statements in i2c_driver.h to 1mS 
//if you are having troubles getting the clock to run.

unsigned char iee_data;
unsigned char iee_addr;
unsigned char nRTC[8];

//Following boilerplate code to use sourceboost i2c library in software emulation mode
//Note: i2c_driver.h was modifed to have 1mS delays for use in 8MHz 16F88 pic+DS1307
////////////////////////////////////////////////////////////////////////////
// i2c master hardware / software mode definition
//
// For i2c hardware support comment out the #define use_ic2_SW line
////////////////////////////////////////////////////////////////////////////
#define use_i2c_SW
//
#if defined use_i2c_SW
////////////////////////////////////////////////////////////////////////////
// i2c software implementation template arguments
////////////////////////////////////////////////////////////////////////////
#define i2c_ARGS	6, PORTA, TRISA, 7, PORTA, TRISA, e_SSPCON1, e_SSPCON2, \
					e_SSPSTAT, e_SSPBUF, e_SSPIF_BIT, e_SSPIF_PIR,			\
					e_BCLIF_BIT, e_BCLIF_PIR, 7, e_SSPADD, (i2c_reset_wdt | i2c_SMP)

// RAM used by the software i2c driver to emulate the equivalent i2c hardware registers					
unsigned short swi2c_SSPCON1@0x40;	// define location for the emulated SSPCON1
unsigned short swi2c_SSPCON2@0x41;	// define location for the emulated SSPCON2
unsigned short swi2c_SSPSTAT@0x42;	// define location for the emulated SSPSTAT
unsigned short swi2c_SSPBUF@0x43;	// define location for the emulated SSPBUF
unsigned short swi2c_SSPIF_PIR@0x44;// define location for the emulated SSPIF_PIR
unsigned short swi2c_BCLIF_PIR@0x45;// define location for the emulated BCLIF_PIR
unsigned short swi2c_SSPADD@0x46;	// define location for the emulated SSPADD

// variables cannot be passed as template arguments. The following constants map to
// the PIC registers and software emalated i2c RAM locations. These constants are
// then used by the templated functions. When changing the address of an emulated
// register the corresponding constant mapping must also be changed.
#define e_SSPCON1	0x40
#define e_SSPCON2	0x41
#define e_SSPSTAT	0x42
#define e_SSPADD	0x43
#define e_SSPBUF	0x44
#define e_SSPIF_PIR	0x45
#define e_BCLIF_PIR	0x46
#define e_SSPIF_BIT	3
#define e_BCLIF_BIT	3					

#endif

//I2C address of DS1307 RTC chip
#define RTCAddress 0xd0

//FUNCTION to write DS1307 RTC data from array nRTC[8]
//INPUTS:	nRTC[0..7] updated with RTC data
//OUTPUTS:	none
//USES: Sourceboost i2c library functions
void write_rtc(void)
{
	unsigned char i;

	i2c_start();
	i2c_write(RTCAddress); // send i2c address					
	i2c_write(0); // send register zero address

	// Write loop
	for (i=0;i<8;i++)
		i2c_write(nRTC[i]);
	i2c_stop();	

}

//FUNCTION to read DS1307 RTC data into array nRTC[8]
//INPUTS:	none
//OUTPUTS:	nRTC[0..7] updated with RTC data
//USES: Sourceboost i2c library functions
void read_rtc(void)
{
	unsigned char i;

	i2c_start();
	i2c_write(RTCAddress); // send RTC address

	i2c_write(0); // send register zero address (seconds/Clock Halt)
	i2c_restart(); // send i2c_restart to indicate a read

	// indicate RTC clock should send data
	i2c_write(RTCAddress | 0x01); // send device address + RD to I2C device

	// read loop
	for (i=0;i<7;i++)
		nRTC[i] = i2c_read(0);	//ack all bytes except for
	nRTC[7] = i2c_read(1);		//final byte

	i2c_stop();	
}

//ASM code to perform pic internal EEPROM write...callable from ASM if required.
//INPUTS: 	iee_addr :- address to write to
//			iee_data :- data to write
void iee_writeenter(void)
{
	asm 
	{

			movf	_iee_data,W
			movwf	_eedata
			movf	_iee_addr,W
			movwf	_eeadr
			bcf		_eecon1,EEPGD
			bsf		_eecon1,WREN
			bcf		_intcon,GIE
			movlw	0x55
			movwf	_eecon2
			movlw	0xAA
			movwf	_eecon2
			bsf		_eecon1,WR
			bsf		_intcon,GIE
iee_write_l:
			btfsc	_eecon1,WR
			goto	iee_write_l
	}
	return;
}

//Function to write to internal eeprom address
//INPUTS: 	addr :- address to write to
//			data :- data to write
void eewrite(unsigned char addr,unsigned char data)
{
	unsigned char n;
	iee_addr = addr;
	iee_data = data;

	iee_writeenter();
}

//Function to read from internal eeprom
//INPUTS: 	addr :- address to read
//RETURNS: 	byte at eeprom address specified
unsigned char eeread(unsigned char addr)
{
	iee_addr = addr;
	asm
	{
			movf	_iee_addr,W
			movf	_iee_addr,W
			movwf	_eeadr
			bcf		_eecon1,EEPGD
			bsf		_eecon1,RD
			movf	_eedata,W
			movwf	_iee_data
	}
	return iee_data;
	
}
