#include <system.h>
#include "rand.h"
#include "dotclock.h"
#include "display.h"
#include "Eprom_Functions.h"

//current display parameters
extern unsigned char n10Hours;
extern unsigned char nHours;
extern unsigned char n10Mins;
extern unsigned char nMins;
//LED layout for each digit in display
unsigned char HOURS10_LIST[6]={LED21, LED22, LED23, LED24, LED25, LED26};
unsigned char HOURS_LIST[9]={LED27,LED28,LED29,LED18,LED19,LED20,LED15,LED16,LED17};
unsigned char MIN10_LIST[6]={LED12,LED14,LED30,LED10,LED11,LED13};
unsigned char MIN_LIST[9]={LED7,LED8,LED9,LED4,LED5,LED6,LED1,LED2,LED3};

//32 bytes to be output
unsigned char nDotList[32];

//misc bits and pieces
unsigned char nDisplayOff=0;
unsigned int nTimeCount=TICK_TIME;
unsigned char nSeconds=1;
unsigned char n20thSeconds=100;
unsigned char nDotPosition=0;
extern unsigned char nRTC[8];

//FUNCTION to read the RTC and update the display parameters
void get_time(void)
{
	read_rtc();
	nMins = nRTC[1] & 0x0f;
	n10Mins = nRTC[1] & 0xf0;
	n10Mins = n10Mins >> 4;
	nHours = nRTC[2] & 0x0f;
	n10Hours = nRTC[2] & 0x30;
	n10Hours = n10Hours>>4;
}

//FUNCTION to Initialise the RTC
void init_display()
{

	nRTC[0] = 0x00;
	nRTC[1] = 0x00;
	nRTC[2] = 0x40;
	nRTC[3] = 3;
	nRTC[4] = 4;
	nRTC[5] = 5;
	nRTC[6] = 99;
	nRTC[7] = 0x90;
	write_rtc();
	display_time();
	
}

//INTERRUPT FUNCTION simply updates the timer and calls refresh_display to determine the next
//byte to send to display chips
void interrupt(void)
{
	tmr1l = 0;
	tmr1h = TMR1_H;
	tmr1l = TMR1_L;
	clear_bit(pir1, TMR1IF);
	
	refresh_display();
}


//FUNCTION to calculate the random positioning of the number of LEDs corresponding to the 
//timer parameters
//INPUTS: n10Hours, nHours, n10Mins, nMins :- display parameters
//OUTPUTS: updated nDotList
//USES: sourceboost random function library
//NOTE: display turned off whilst update is being done
void display_time(void)
{
	unsigned char i,j,k,l;
	
	//bound parameters to make sure bogus values don't send algorithm into la la land
	if (n10Hours>5) n10Hours = 5;
	if (nHours>9) nHours=9;
	if (n10Mins>5) n10Mins = 5;
	if (nMins>9) nMins = 9;
	
	//10Hours
	nDisplayOff = 1;
	k = n10Hours;				//number of 10Leds to set
	for(i=0;i<6;i++) nDotList[i]=HOURS10_LIST[i];
	j = 6-k;
	for(i=0;i<j;i++)
	{
		k = rand() % 6;			//pick a random position
		if(nDotList[k]==OFFVAL)	//already cleared
			i--;				//so try again till we've cleared j items
		else
			nDotList[k] = OFFVAL;//clear this item
	}
	
	//hours
	k = nHours;					//number of leds to set
	for(i=6;i<15;i++) nDotList[i]=HOURS_LIST[i-6];
	j = 15-nHours;
	for(i=6;i<j;i++)
	{
		k = rand() % 9;			//pick a random position
		k = k + 6;
		if(nDotList[k]==OFFVAL)	//already cleared
			i--;				//so try again till we've cleared j items
		else
			nDotList[k] = OFFVAL;//clear this item
	}
	
	//10mins
	k = n10Mins;
	for(i=15;i<21;i++) nDotList[i]=MIN10_LIST[i-15];
	j = 21-k;
	for(i=15;i<j;i++)
	{
		k = rand() % 6;			//pick a random position
		k = k + 15;
		if(nDotList[k]==OFFVAL)	//already cleared
			i--;				//so try again till we've cleared j items
		else
			nDotList[k] = OFFVAL;//clear this item
	}
	
	//mins
	k = nMins;
	for(i=21;i<30;i++) nDotList[i]=MIN_LIST[i-21];
	j = 30-k;
	for(i=21;i<j;i++)
	{
		k = rand() % 9;		//pick a random position
		k = k + 21;
		if(nDotList[k]==OFFVAL)	//already cleared
			i--;			//so try again till we've cleared j items
		else
			nDotList[k] = OFFVAL;//clear this item
	}	
	nDotList[30] = 0xfe;
	nDotList[31] = 0xfe;
	nDisplayOff = 0;
}

//FUNCTION to pick next value out of LED list and send to display chips
void refresh_display(void)
{
	unsigned char i,j;
	if(nDisplayOff==1) 
	{
		portb = OFFVAL;
		nDotPosition = 0;
		return;
	}
	nDotPosition++;
	if (nDotPosition==32) nDotPosition=0;
	j = nDotList[nDotPosition];
	portb = j;
}